#include <cmath>

#include "GLpoint3f.h"

GLpoint3f operator-(const GLpoint3f& p1, const GLpoint3f& p2)
{
   GLpoint3f q(p1.x - p2.x, p1.y - p2.y, p1.z - p2.z);
   return q;
}

GLpoint3f operator+(const GLpoint3f& p1, const GLpoint3f& p2)
{
   GLpoint3f q(p1.x + p2.x, p1.y + p2.y, p1.z + p2.z);
   return q;
}

GLpoint3f operator*(const GLpoint3f& p, const GLfloat& s)
{
   GLpoint3f q(p.x * s, p.y * s, p.z * s);
   return q;
}

void operator*=(GLpoint3f& p, const GLfloat& s)
{
   p.x *= s;
   p.y *= s;
   p.z *= s;
}

GLpoint3f GLpoint3f::cross(const GLpoint3f& p) const
{
   GLpoint3f q(y * p.z - z * p.y, z * p.x - x * p.z, x * p.y - y * p.x);
   return q;
}

GLfloat GLpoint3f::length2()
{
   return x*x + y*y + z*z;
}

GLfloat GLpoint3f::length()
{
   return sqrt(x*x + y*y + z*z);
}

void GLpoint3f::normalise()
{
   GLfloat l = length();
   x /= l;
   y /= l;
   z /= l;
}

GLfloat get_area(const GLpoint3f& v1, const GLpoint3f& v2, const GLpoint3f& v3)
{
   return ((v2 - v1).cross(v3 - v1)).length() * 0.5f;
}