#pragma once
#include <list>

#include "GLRGBColour.h"
#include "GLpoint3f.h"
#include "3DS Loader/IMesh.h"

class Mesh : public IMesh
{
public:
   static const unsigned int max_patches = 8192, max_vertices = 8192;
   class Patch
   {
   public:
      unsigned int vertex_ind[3]; // The 3 here represents the fact that we're using triangles. In some ways, quads would be easier!
      GLRGBColour radiosity, unsent_radiosity;
      GLfloat area;
   };

   unsigned int n_vertices, n_patches;
   GLpoint3f vertices[max_vertices];
   GLRGBColour colour;
   GLfloat reflectance; // reflectance could be built into colour but it's handy to have it.
   Patch patches[max_patches]; // Could use a list here but this is simpler and it's handy to have random access at times.
   std::list<unsigned int> *patch_ind;
   GLRGBColour *vertex_radiosity;

   Mesh(void);
   ~Mesh(void);
   void subdivide(void);
   void initialize(const unsigned int n_subdivisions, const GLRGBColour& colour, const GLfloat& reflectance, const GLRGBColour& emission);
   void update_patch_ind(void);
   void update_vertex_radiosities(void);
// These last four methods are needed because of the IMesh inheritance.
   void setFace(int index, int a, int b, int c);
   void setVertex(int index, float x, float y, float z);
   int getVertCnt(void);
   int getFaceCnt(void);
};